@@ -38,7 +38,7 @@ import sys
 import traceback
 
 from envstack import __version__, config
-from envstack.env import build_sources, expandvars, export, load_environ, trace_var
+from envstack.env import clear, export, load_environ, resolve_environ, trace_var
 from envstack.wrapper import run_command
 
 
@@ -90,6 +90,11 @@ def parse_args():
         metavar="PLATFORM",
         help="platform to resolve variables for (linux, darwin, windows)",
     )
+    parser.add_argument(
+        "--scope",
+        metavar="SCOPE",
+        help="search scope for environment stack files",
+    )
     parser.add_argument(
         "-r",
         "--resolve",
@@ -115,40 +120,48 @@ def parse_args():
     return args, args_after_dash
 
 
+def whichenv():
+    """Entry point for the whichenv command line tool"""
+    from envstack.util import findenv
+
+    var_name = sys.argv[1]
+    paths = findenv(var_name)
+    for path in paths:
+        print("{0}: {1}".format(var_name, path))
+
+
 def main():
     """Main thread."""
     args, command = parse_args()
 
     try:
-        if args.resolve is not None:
-            if len(args.resolve) == 0:
-                args.resolve = load_environ(args.namespace).keys()
-            env = load_environ(args.namespace, platform=args.platform)
-            for resolve in args.resolve:
-                var = env.get(resolve)
-                val = expandvars(var, env, recursive=True)
-                print(f"{resolve}={val}")
+        if command:
+            return run_command(command, args.namespace)
+        elif args.resolve is not None:
+            resolved = resolve_environ(
+                load_environ(args.namespace, platform=args.platform)
+            )
+            keys = args.resolve or resolved.keys()
+            for key in sorted(keys):
+                val = resolved.get(key)
+                print(f"{key}={val}")
         elif args.trace is not None:
             if len(args.trace) == 0:
                 args.trace = load_environ(args.namespace).keys()
             for trace in args.trace:
-                path = trace_var(args.namespace, trace)
+                path = trace_var(*args.namespace, var=trace)
                 print("{0}: {1}".format(trace, path))
         elif args.sources:
-            sources = build_sources(args.namespace)
-            for source in sources:
-                print(source)
+            env = load_environ(args.namespace, platform=args.platform)
+            for source in env.sources:
+                print(source.path)
         elif args.clear:
-            print(export(args.namespace, config.SHELL, clear=True))
+            print(clear(args.namespace, config.SHELL))
         elif args.export:
             print(export(args.namespace, config.SHELL))
-        elif command:
-            return run_command(command, args.namespace)
         else:
-            env = load_environ(
-                args.namespace, environ=None, platform=args.platform, includes=True
-            )
-            for k, v in env.items():
+            env = load_environ(args.namespace, platform=args.platform)
+            for k, v in sorted(env.items(), key=lambda x: str(x[0])):
                 print(f"{k}={v}")
 
     except KeyboardInterrupt:
