@@ -50,17 +50,18 @@ def detect_shell():
         else:
             return "unknown"
     else:
-        shell = os.environ.get("SHELL")
+        shell = os.environ.get("SHELL", "/bin/bash")
         if shell:
-            return shell.split("/")[-1]
+            return shell
         else:
-            return "unknown"
+            return "/usr/bin/bash"
 
 
 DEBUG = os.getenv("DEBUG")
-DEFAULT_NAMESPACE = os.getenv("DEFAULT_ENV_STACK", "stack")
+DEFAULT_NAMESPACE = os.getenv("DEFAULT_ENV_STACK", "default")
 ENV = os.getenv("ENV", "prod")
 HOME = os.getenv("HOME")
+IGNORE_MISSING = bool(os.getenv("IGNORE_MISSING", 1))
 LOG_LEVEL = os.environ.get("LOG_LEVEL", "INFO")
 ON_POSIX = "posix" in sys.builtin_module_names
 PLATFORM = platform.system().lower()
@@ -68,23 +69,20 @@ PYTHON_VERSION = sys.version_info[0]
 SHELL = detect_shell()
 USERNAME = os.getenv("USERNAME", os.getenv("USER"))
 
-# set some default environment variables
+# set some default environment values
 DEFAULT_ENV = {
     "ENV": ENV,
     "HOME": HOME,
     "PLATFORM": PLATFORM,
+    "ROOT": os.getenv(
+        "ROOT",
+        {
+            "darwin": "{HOME}/Library/Application Support/pipe",
+            "linux": "{HOME}/.local/pipe",
+            "windows": "C:\\ProgramData\\pipe",
+        }
+        .get(PLATFORM)
+        .format(**locals()),
+    ),
     "USER": USERNAME,
 }
-
-# default location of envstack files
-# NOTE: the siteconf sitecustomize.py module may override this
-DEFAULT_ENV_DIR = os.getenv(
-    "DEFAULT_ENV_DIR",
-    {
-        "darwin": "{HOME}/Library/Application Support/envstack",
-        "linux": "{HOME}/.local/envstack",
-        "windows": "C:\\ProgramData\\envstack",
-    }
-    .get(PLATFORM)
-    .format(**DEFAULT_ENV),
-)
