@@ -40,15 +40,37 @@ import subprocess
 import traceback
 
 from envstack import config, logger
-from envstack.env import encode, expandvars, load_environ
+from envstack.env import load_environ, resolve_environ
+from envstack.util import encode, evaluate_modifiers
 
 
-def to_args(cmd):
-    """Converts a command line string to an arg list to be passed to
-    subprocess.Popen that preserves args with quotes."""
+def to_args(cmd: str):
+    """
+    Converts a command line string to an arg list to be passed to
+    subprocess.Popen that preserves args with quotes.
+
+    :param cmd: command line string.
+    """
     return shlex.split(cmd)
 
 
+def shell_join(args):
+    """
+    Joins a list of arguments into a single quoted shell string.
+
+    :param args: list of arguments.
+    :returns: shell string.
+    """
+    argstr = ".".join(args)
+    if '"' in argstr or "'" in argstr:
+        try:
+            return shlex.join(args)
+        except AttributeError:
+            return " ".join(shlex.quote(arg) for arg in args)
+    else:
+        return " ".join(args)
+
+
 class Wrapper(object):
     """Wrapper class for executables. Subprocessed with preconfigured environment.
 
@@ -74,8 +96,8 @@ class Wrapper(object):
     def __init__(self, namespace, args=[]):
         """Initializes the wrapper with the given namespace and args.
 
-        :param namespace: namespace
-        :param args: command line arguments
+        :param namespace: environment stack namespace.
+        :param args: command line arguments.
         """
         super(Wrapper, self).__init__()
         self.args = args
@@ -98,14 +120,12 @@ class Wrapper(object):
             process = subprocess.Popen(
                 args=command,
                 bufsize=0,
-                env=env,
+                env=encode(env),
                 shell=self.shell,
             )
-
         except Exception:
             traceback.print_exc()
             exitcode = 1
-
         else:
             stdout, stderr = process.communicate()
             while stdout and stderr:
@@ -121,7 +141,7 @@ class Wrapper(object):
 
     def get_subprocess_command(self, env):
         """Returns the command to be passed to the subprocess."""
-        cmd = expandvars(self.executable(), env, recursive=True)
+        cmd = evaluate_modifiers(self.executable(), env)
         args = self.get_subprocess_args(cmd)
         return " ".join([cmd] + args)
 
@@ -131,7 +151,7 @@ class Wrapper(object):
         is called on the wrapper.
         """
         env = os.environ.copy()
-        env.update(encode(self.env))
+        env.update(resolve_environ(self.env))
         return env
 
 
@@ -142,14 +162,14 @@ class CommandWrapper(Wrapper):
         """
         Initializes the command wrapper with the given namespace and args, e.g.:
 
-            >>> cmd = CommandWrapper('stack', ['ls', '-al'])
+            >>> cmd = CommandWrapper(stack, ['ls', '-al'])
             >>> print(cmd.executable())
             ls
             >>> print(cmd.args)
             ['-al']
 
-        :param namespace: stack namespace (default: 'stack')
-        :param args: command and arguments as a list
+        :param namespace: environment stack name (default: 'default').
+        :param args: command and arguments as a list.
         """
         super(CommandWrapper, self).__init__(namespace, args)
         self.cmd = args
@@ -167,25 +187,34 @@ class ShellWrapper(CommandWrapper):
         Initializes the command wrapper with the given namespace and args,
         replacing the original command with the shell command, e.g.:
 
-            >>> cmd = ShellWrapper('stack', ['ls', '-l'])
+            >>> cmd = ShellWrapper(stack, ['ls', '-l'])
             >>> print(cmd.executable())
             bash
             >>> print(cmd.args)
             ['-i', '-c', 'ls -l']
 
-        :param namespace: stack namespace (default: 'stack')
-        :param args: command and arguments as a list
+        :param namespace: environment stack name (default: 'default').
+        :param args: command and arguments as a list.
+        :param interactive: run the command in an interactive shell (default: True).
         """
         super(ShellWrapper, self).__init__(namespace, args)
+        self.interactive = self.get_interactive()
 
-    def get_subprocess_command(self, env):
+    def get_interactive(self):
+        """Returns whether to run the command in an interactive shell."""
+        return bool(int(os.getenv("INTERACTIVE", 1)))
+
+    def get_subprocess_command(self, env: dict):
         """Returns the command to be passed to the shell in a subprocess."""
-        self.cmd = expandvars(self.cmd, env, recursive=True)
         if re.search(r"\$\w+", self.cmd):
-            return f'{config.SHELL} -i -c "{self.cmd}"'
+            if self.interactive:
+                return f'{config.SHELL} -i -c "{self.cmd}"'
+            return f'{config.SHELL} -c "{self.cmd}"'
         else:
             escaped_command = shlex.quote(self.cmd)
-            return f"{config.SHELL} -i -c {escaped_command}"
+            if self.interactive:
+                return f"{config.SHELL} -i -c {escaped_command}"
+            return f"{config.SHELL} -c {escaped_command}"
 
     def executable(self):
         """Returns the shell command to run the original command."""
@@ -200,14 +229,14 @@ class CmdWrapper(CommandWrapper):
         Initializes the command wrapper with the given namespace and args,
         replacing the original command with the shell command, e.g.:
 
-            >>> cmd = CmdWrapper('stack', ['dir'])
+            >>> cmd = CmdWrapper(stack, ['dir'])
             >>> print(cmd.executable())
             cmd
             >>> print(cmd.args)
             ['/c', 'dir']
 
-        :param namespace: stack namespace (default: 'stack')
-        :param args: command and arguments as a list
+        :param namespace: environment stack name (default: 'default').
+        :param args: command and arguments as a list.
         """
         super(CmdWrapper, self).__init__(namespace, args)
         self.args = ["/c", self.cmd]
@@ -223,7 +252,7 @@ class CmdWrapper(CommandWrapper):
         return self.cmd
 
 
-def run_command(command, namespace=config.DEFAULT_NAMESPACE):
+def run_command(command: str, namespace: str = config.DEFAULT_NAMESPACE):
     """
     Runs a given command with the given stack namespace.
 
@@ -233,15 +262,22 @@ def run_command(command, namespace=config.DEFAULT_NAMESPACE):
 
         >>> run_command(['ls', '-l'], 'my-stack')
 
-    :param command: command to run as a list of arguments
-    :param namespace: environment stack namespace (default: 'stack')
-    :returns: exit code
+     - Automatically detects the shell to use.
+     - Converts {VAR} to $VAR for bash, sh, zsh, and %VAR% for cmd.
+
+    :param command: command to run as a list of arguments.
+    :param namespace: environment stack name (default: 'default').
+    :param interactive: run the command in an interactive shell (default: True).
+    :returns: command exit code
     """
     logger.setup_stream_handler()
-    if config.SHELL in ["bash", "sh", "zsh"]:
-        cmd = ShellWrapper(namespace, shlex.join(command))
-    elif config.SHELL in ["cmd"]:
-        cmd = CmdWrapper(namespace, " ".join(command))
+    shellname = os.path.basename(config.SHELL)
+    if shellname in ["bash", "sh", "zsh"]:
+        command = re.sub(r"\{(\w+)\}", r"${\1}", shell_join(command))
+        cmd = ShellWrapper(namespace, command)
+    elif shellname in ["cmd"]:
+        command = re.sub(r"\{(\w+)\}", r"%\1%", " ".join(command))
+        cmd = CmdWrapper(namespace, command)
     else:
         cmd = CommandWrapper(namespace, command)
     return cmd.launch()
